@@ -15,12 +15,16 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
+
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
+
 #include <assert.h>
 #include <gtk/gtk.h>
 #include <math.h>
 #include <string.h>
 
-#include "config.h"
 #include "intl.h"
 #include "object.h"
 #include "element.h"
@@ -28,6 +32,7 @@
 #include "render.h"
 #include "attributes.h"
 #include "widgets.h"
+#include "properties.h"
 
 #include "pixmaps/attribute.xpm"
 
@@ -39,14 +44,12 @@
 #define TEXT_BORDER_WIDTH_X 1.0
 #define TEXT_BORDER_WIDTH_Y 0.5
 
-typedef struct _AttributeState AttributeState;
 typedef struct _Attribute Attribute;
-typedef struct _AttributePropertiesDialog AttributePropertiesDialog;
 
 struct _AttributeState {
   ObjectState obj_state;
 
-  gchar *name;
+  utfchar *name;
   real name_width;
 
   gboolean key;
@@ -62,8 +65,9 @@ struct _AttributeState {
 struct _Attribute {
   Element element;
 
-  Font *font;
-  gchar *name;
+  DiaFont *font;
+  real font_height;
+  utfchar *name;
   real name_width;
 
   ConnectionPoint connections[8];
@@ -76,21 +80,6 @@ struct _Attribute {
   real border_width;
   Color border_color;
   Color inner_color;
- 
-  AttributePropertiesDialog *properties_dialog;
-};
-
-struct _AttributePropertiesDialog {
-  GtkWidget *vbox;
-
-  GtkEntry *name;
-  GtkToggleButton *key;
-  GtkToggleButton *weakkey;
-  GtkToggleButton *derived;
-  GtkToggleButton *multivalue;
-  GtkSpinButton *border_width;
-  DiaColorSelector *fg_color;
-  DiaColorSelector *bg_color;
 };
 
 static real attribute_distance_from(Attribute *attribute, Point *point);
@@ -108,15 +97,17 @@ static Object *attribute_create(Point *startpoint,
 			      Handle **handle2);
 static void attribute_destroy(Attribute *attribute);
 static Object *attribute_copy(Attribute *attribute);
-static GtkWidget *attribute_get_properties(Attribute *attribute);
-static ObjectChange *attribute_apply_properties(Attribute *attribute);
+static PropDescription *
+attribute_describe_props(Attribute *attribute);
+static void
+attribute_get_props(Attribute *attribute, GPtrArray *props);
+static void
+attribute_set_props(Attribute *attribute, GPtrArray *props);
 
 static void attribute_save(Attribute *attribute, ObjectNode obj_node,
 			   const char *filename);
 static Object *attribute_load(ObjectNode obj_node, int version,
 			      const char *filename);
-static AttributeState *attribute_get_state(Attribute *Attribute);
-static void attribute_set_state(Attribute *Attribute, AttributeState *state);
 
 static ObjectTypeOps attribute_type_ops =
 {
@@ -144,141 +135,73 @@ static ObjectOps attribute_ops = {
   (CopyFunc)            attribute_copy,
   (MoveFunc)            attribute_move,
   (MoveHandleFunc)      attribute_move_handle,
-  (GetPropertiesFunc)   attribute_get_properties,
-  (ApplyPropertiesFunc) attribute_apply_properties,
-  (ObjectMenuFunc)      NULL
+  (GetPropertiesFunc)   object_create_props_dialog,
+  (ApplyPropertiesFunc) object_apply_props_from_dialog,
+  (ObjectMenuFunc)      NULL,
+  (DescribePropsFunc)   attribute_describe_props,
+  (GetPropsFunc)        attribute_get_props,
+  (SetPropsFunc)        attribute_set_props,
 };
 
-static ObjectChange *
-attribute_apply_properties(Attribute *attribute)
+static PropDescription attribute_props[] = {
+  ELEMENT_COMMON_PROPERTIES,
+  { "name", PROP_TYPE_STRING, PROP_FLAG_VISIBLE,
+    N_("Name:"), NULL, NULL },
+  { "key", PROP_TYPE_BOOL, PROP_FLAG_VISIBLE,
+    N_("Key:"), NULL, NULL },
+  { "weakkey", PROP_TYPE_BOOL, PROP_FLAG_VISIBLE,
+    N_("Weak key:"), NULL, NULL },
+  { "derived", PROP_TYPE_BOOL, PROP_FLAG_VISIBLE,
+    N_("Derived:"), NULL, NULL },
+  { "multivalue", PROP_TYPE_BOOL, PROP_FLAG_VISIBLE,
+    N_("Multivalue:"), NULL, NULL },
+  PROP_STD_LINE_WIDTH,
+  PROP_STD_LINE_COLOUR,
+  PROP_STD_FILL_COLOUR,
+  PROP_STD_TEXT_FONT,
+  PROP_STD_TEXT_HEIGHT,
+  PROP_DESC_END
+};
+
+static PropDescription *
+attribute_describe_props(Attribute *attribute)
 {
-  ObjectState *old_state;
-  AttributePropertiesDialog *prop_dialog;
+  if (attribute_props[0].quark == 0)
+    prop_desc_list_calculate_quarks(attribute_props);
+  return attribute_props;
+}
 
-  prop_dialog = attribute->properties_dialog;
+static PropOffset attribute_offsets[] = {
+  ELEMENT_COMMON_PROPERTIES_OFFSETS,
+  { "name", PROP_TYPE_STRING, offsetof(Attribute, name) },
+  { "key", PROP_TYPE_BOOL, offsetof(Attribute, key) },
+  { "weakkey", PROP_TYPE_BOOL, offsetof(Attribute, weakkey) },
+  { "derived", PROP_TYPE_BOOL, offsetof(Attribute, derived) },
+  { "multivalue", PROP_TYPE_BOOL, offsetof(Attribute, multivalue) },
+  { "line_width", PROP_TYPE_REAL, offsetof(Attribute, border_width) },
+  { "line_colour", PROP_TYPE_COLOUR, offsetof(Attribute, border_color) },
+  { "fill_colour", PROP_TYPE_COLOUR, offsetof(Attribute, inner_color) },
+  { "text_font", PROP_TYPE_FONT, offsetof(Attribute, font) },
+  { "text_height", PROP_TYPE_REAL, offsetof(Attribute, font_height) },
+  { NULL, 0, 0}
+};
 
-  old_state = (ObjectState *)attribute_get_state(attribute);
 
-  attribute->border_width = gtk_spin_button_get_value_as_float(prop_dialog->border_width);
-  dia_color_selector_get_color(prop_dialog->fg_color, &attribute->border_color);
-  dia_color_selector_get_color(prop_dialog->bg_color, &attribute->inner_color);
-  g_free(attribute->name);
-  attribute->name = g_strdup(gtk_entry_get_text(prop_dialog->name));
-  attribute->key = prop_dialog->key->active;
-  attribute->weakkey = prop_dialog->weakkey->active;
-  attribute->derived = prop_dialog->derived->active;
-  attribute->multivalue = prop_dialog->multivalue->active;
-
-  attribute->name_width =
-    font_string_width(attribute->name, attribute->font, FONT_HEIGHT);
+static void
+attribute_get_props(Attribute *attribute, GPtrArray *props)
+{
+  object_get_props_from_offsets(&attribute->element.object, 
+                                attribute_offsets, props);
+}
 
+static void
+attribute_set_props(Attribute *attribute, GPtrArray *props)
+{
+  object_set_props_from_offsets(&attribute->element.object, 
+                                attribute_offsets, props);
   attribute_update_data(attribute);
-
-  return new_object_state_change((Object *)attribute, old_state,
-				 (GetStateFunc)attribute_get_state,
-				 (SetStateFunc)attribute_set_state);
 }
 
-static GtkWidget *
-attribute_get_properties(Attribute *attribute)
-{
-  AttributePropertiesDialog *prop_dialog;
-  GtkWidget *vbox;
-  GtkWidget *hbox;
-  GtkWidget *label;
-  GtkWidget *color;
-  GtkWidget *border_width;
-  GtkWidget *entry;
-  GtkWidget *checkbox;
-  GtkAdjustment *adj;
-
-  if (attribute->properties_dialog == NULL) {
-  
-    prop_dialog = g_new(AttributePropertiesDialog, 1);
-    attribute->properties_dialog = prop_dialog;
-
-    vbox = gtk_vbox_new(FALSE, 5);
-    gtk_object_ref(GTK_OBJECT(vbox));
-    gtk_object_sink(GTK_OBJECT(vbox));
-    prop_dialog->vbox = vbox;
-
-    hbox = gtk_hbox_new(FALSE, 5);
-    label = gtk_label_new(_("Name:"));
-    entry = gtk_entry_new();
-    prop_dialog->name = GTK_ENTRY(entry);
-    gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, TRUE, 0);
-    gtk_box_pack_start (GTK_BOX (hbox), entry, TRUE, TRUE, 0);
-    gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, TRUE, 0);
-
-    hbox = gtk_hbox_new(FALSE, 5);
-    checkbox = gtk_check_button_new_with_label(_("Key"));
-    prop_dialog->key = GTK_TOGGLE_BUTTON(checkbox);
-    gtk_box_pack_start (GTK_BOX (hbox), checkbox, TRUE, TRUE, 0);
-    gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, TRUE, 0);
-
-    hbox = gtk_hbox_new(FALSE, 5);
-    checkbox = gtk_check_button_new_with_label(_("Weak key"));
-    prop_dialog->weakkey = GTK_TOGGLE_BUTTON(checkbox);
-    gtk_box_pack_start (GTK_BOX (hbox), checkbox, TRUE, TRUE, 0);
-    gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, TRUE, 0);
-
-    hbox = gtk_hbox_new(FALSE, 5);
-    checkbox = gtk_check_button_new_with_label(_("Derived"));
-    prop_dialog->derived = GTK_TOGGLE_BUTTON(checkbox);
-    gtk_box_pack_start (GTK_BOX (hbox), checkbox, TRUE, TRUE, 0);
-    gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, TRUE, 0);
-
-    hbox = gtk_hbox_new(FALSE, 5);
-    checkbox = gtk_check_button_new_with_label(_("Multivalue"));
-    prop_dialog->multivalue = GTK_TOGGLE_BUTTON(checkbox);
-    gtk_box_pack_start (GTK_BOX (hbox), checkbox, TRUE, TRUE, 0);
-    gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, TRUE, 0);
-
-    hbox = gtk_hbox_new(FALSE, 5);
-    label = gtk_label_new(_("Border width:"));
-    gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, TRUE, 0);
-    adj = (GtkAdjustment *) gtk_adjustment_new(0.1, 0.00, 10.0, 0.01, 0.0, 0.0);
-    border_width = gtk_spin_button_new(adj, 1.0, 2);
-    gtk_spin_button_set_wrap(GTK_SPIN_BUTTON(border_width), TRUE);
-    gtk_spin_button_set_numeric(GTK_SPIN_BUTTON(border_width), TRUE);
-    prop_dialog->border_width = GTK_SPIN_BUTTON(border_width);
-    gtk_box_pack_start(GTK_BOX (hbox), border_width, TRUE, TRUE, 0);
-    gtk_box_pack_start (GTK_BOX (vbox), hbox, TRUE, TRUE, 0);
-
-    hbox = gtk_hbox_new(FALSE, 5);
-    label = gtk_label_new(_("Foreground color:"));
-    gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, TRUE, 0);
-    color = dia_color_selector_new();
-    prop_dialog->fg_color = DIACOLORSELECTOR(color);
-    gtk_box_pack_start (GTK_BOX (hbox), color, TRUE, TRUE, 0);
-    gtk_box_pack_start (GTK_BOX (vbox), hbox, TRUE, TRUE, 0);
-
-    hbox = gtk_hbox_new(FALSE, 5);
-    label = gtk_label_new(_("Background color:"));
-    gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, TRUE, 0);
-    color = dia_color_selector_new();
-    prop_dialog->bg_color = DIACOLORSELECTOR(color);
-    gtk_box_pack_start (GTK_BOX (hbox), color, TRUE, TRUE, 0);
-    gtk_box_pack_start (GTK_BOX (vbox), hbox, TRUE, TRUE, 0);
-
-    gtk_widget_show_all (vbox);
-  }
-
-  prop_dialog = attribute->properties_dialog;
-    
-  gtk_entry_set_text(prop_dialog->name, attribute->name);
-  gtk_toggle_button_set_active(prop_dialog->key, attribute->key);
-  gtk_toggle_button_set_active(prop_dialog->weakkey, attribute->weakkey);
-  gtk_toggle_button_set_active(prop_dialog->derived, attribute->derived);
-  gtk_toggle_button_set_active(prop_dialog->multivalue, attribute->multivalue);
-  gtk_spin_button_set_value(prop_dialog->border_width, attribute->border_width);
-  dia_color_selector_set_color(prop_dialog->fg_color, &attribute->border_color);
-  dia_color_selector_set_color(prop_dialog->bg_color, &attribute->inner_color);
-  
-  return prop_dialog->vbox;
-}
-
-
 static real
 attribute_distance_from(Attribute *attribute, Point *point)
 {
@@ -356,10 +279,10 @@ attribute_draw(Attribute *attribute, Renderer *renderer)
   }
 
   p.x = elem->corner.x + elem->width / 2.0;
-  p.y = elem->corner.y + (elem->height - FONT_HEIGHT)/2.0 +
-         font_ascent(attribute->font, FONT_HEIGHT);
+  p.y = elem->corner.y + (elem->height - attribute->font_height)/2.0 +
+         font_ascent(attribute->font, attribute->font_height);
 
-  renderer->ops->set_font(renderer,  attribute->font, FONT_HEIGHT);
+  renderer->ops->set_font(renderer,  attribute->font, attribute->font_height);
   renderer->ops->draw_string(renderer, attribute->name, 
 			     &p, ALIGN_CENTER, 
 			     &color_black);
@@ -371,7 +294,7 @@ attribute_draw(Attribute *attribute, Renderer *renderer)
     } else {
       renderer->ops->set_linestyle(renderer, LINESTYLE_SOLID);
     }
-    width = font_string_width(attribute->name, attribute->font, FONT_HEIGHT);
+    width = font_string_width(attribute->name, attribute->font, attribute->font_height);
     start.x = center.x - width / 2;
     start.y = center.y + 0.4;
     end.x = center.x + width / 2;
@@ -384,12 +307,16 @@ static void
 attribute_update_data(Attribute *attribute)
 {
   Element *elem = &attribute->element;
-  Object *obj = (Object *) attribute;
+  Object *obj = &elem->object;
   Point center;
+  ElementBBExtras *extra = &elem->extra_spacing;
   real half_x, half_y;
 
+  attribute->name_width =
+    font_string_width(attribute->name, attribute->font, attribute->font_height);
+
   elem->width = attribute->name_width + 2*TEXT_BORDER_WIDTH_X;
-  elem->height = FONT_HEIGHT + 2*TEXT_BORDER_WIDTH_Y;
+  elem->height = attribute->font_height + 2*TEXT_BORDER_WIDTH_Y;
 
   center.x = elem->corner.x + elem->width / 2.0;
   center.y = elem->corner.y + elem->height / 2.0;
@@ -415,12 +342,8 @@ attribute_update_data(Attribute *attribute)
   attribute->connections[7].pos.x = center.x + half_x;
   attribute->connections[7].pos.y = center.y + half_y;
 
+  extra->border_trans = attribute->border_width/2.0;
   element_update_boundingbox(elem);
-  /* fix boundingattribute for line_width: */
-  obj->bounding_box.top -= attribute->border_width/2;
-  obj->bounding_box.left -= attribute->border_width/2;
-  obj->bounding_box.bottom += attribute->border_width/2;
-  obj->bounding_box.right += attribute->border_width/2;
 
   obj->position = elem->corner;
 
@@ -428,43 +351,6 @@ attribute_update_data(Attribute *attribute)
   
 }
 
-static AttributeState *attribute_get_state(Attribute *attribute)
-{
-  AttributeState *state = g_new(AttributeState, 1);
-
-  state->obj_state.free = NULL;
-
-  state->name = g_strdup(attribute->name);
-  state->name_width = attribute->name_width;
-  state->key = attribute->key;
-  state->weakkey = attribute->weakkey;
-  state->derived = attribute->derived;
-  state->multivalue = attribute->multivalue;
-  state->border_width = attribute->border_width;
-  state->border_color = attribute->border_color;
-  state->inner_color = attribute->inner_color;
-
-  return state;
-}
-
-static void attribute_set_state(Attribute *attribute, AttributeState *state)
-{
-  g_free(attribute->name);
-  attribute->name = g_strdup(state->name);
-  attribute->name_width = state->name_width;
-  attribute->key = state->key;
-  attribute->weakkey = state->weakkey;
-  attribute->derived = state->derived;
-  attribute->multivalue = state->multivalue;
-  attribute->border_width = state->border_width;
-  attribute->border_color = state->border_color;
-  attribute->inner_color = state->inner_color;
-
-  g_free(state);
-
-  attribute_update_data(attribute);
-}
-
 static Object *
 attribute_create(Point *startpoint,
 	       void *user_data,
@@ -476,9 +362,9 @@ attribute_create(Point *startpoint,
   Object *obj;
   int i;
 
-  attribute = g_malloc(sizeof(Attribute));
+  attribute = g_malloc0(sizeof(Attribute));
   elem = &attribute->element;
-  obj = (Object *) attribute;
+  obj = &elem->object;
   
   obj->type = &attribute_type;
   obj->ops = &attribute_ops;
@@ -487,8 +373,6 @@ attribute_create(Point *startpoint,
   elem->width = DEFAULT_WIDTH;
   elem->height = DEFAULT_WIDTH;
 
-  attribute->properties_dialog = NULL;
-
   attribute->border_width =  attributes_get_default_linewidth();
   attribute->border_color = attributes_get_foreground();
   attribute->inner_color = attributes_get_background();
@@ -505,11 +389,18 @@ attribute_create(Point *startpoint,
   attribute->weakkey = FALSE;
   attribute->derived = FALSE;
   attribute->multivalue = FALSE;
-  attribute->font = font_getfont("Courier");
+  /* choose default font name for your locale. see also font_data structure
+     in lib/font.c. if "Courier" works for you, it would be better.  */
+  attribute->font = font_getfont(_("Courier"));
+  attribute->font_height = FONT_HEIGHT;
+#ifdef GTK_DOESNT_TALK_UTF8_WE_DO
+  attribute->name = charconv_local8_to_utf8 (_("Attribute"));
+#else
   attribute->name = g_strdup(_("Attribute"));
+#endif
 
   attribute->name_width =
-    font_string_width(attribute->name, attribute->font, FONT_HEIGHT);
+    font_string_width(attribute->name, attribute->font, attribute->font_height);
 
   attribute_update_data(attribute);
 
@@ -519,16 +410,12 @@ attribute_create(Point *startpoint,
 
   *handle1 = NULL;
   *handle2 = obj->handles[0];
-  return (Object *)attribute;
+  return &attribute->element.object;
 }
 
 static void
 attribute_destroy(Attribute *attribute)
 {
-  if (attribute->properties_dialog != NULL) {
-    gtk_widget_destroy(attribute->properties_dialog->vbox);
-    g_free(attribute->properties_dialog);
-  }
   element_destroy(&attribute->element);
   g_free(attribute->name);
 }
@@ -543,9 +430,9 @@ attribute_copy(Attribute *attribute)
   
   elem = &attribute->element;
   
-  newattribute = g_malloc(sizeof(Attribute));
+  newattribute = g_malloc0(sizeof(Attribute));
   newelem = &newattribute->element;
-  newobj = (Object *) newattribute;
+  newobj = &newelem->object;
 
   element_copy(elem, newelem);
 
@@ -562,6 +449,7 @@ attribute_copy(Attribute *attribute)
   }
 
   newattribute->font = attribute->font;
+  newattribute->font_height = attribute->font_height;
   newattribute->name = strdup(attribute->name);
   newattribute->name_width = attribute->name_width;
 
@@ -570,9 +458,7 @@ attribute_copy(Attribute *attribute)
   newattribute->derived = attribute->derived;
   newattribute->multivalue = attribute->multivalue;
 
-  newattribute->properties_dialog = NULL;
-
-  return (Object *)newattribute;
+  return &newattribute->element.object;
 }
 
 
@@ -598,6 +484,10 @@ attribute_save(Attribute *attribute, ObjectNode obj_node,
 		   attribute->derived);
   data_add_boolean(new_attribute(obj_node, "multivalued"),
 		   attribute->multivalue);
+  data_add_font (new_attribute (obj_node, "font"),
+		 attribute->font);
+  data_add_real(new_attribute(obj_node, "font_height"),
+		attribute->font_height);
 }
 
 static Object *attribute_load(ObjectNode obj_node, int version,
@@ -609,17 +499,15 @@ static Object *attribute_load(ObjectNode obj_node, int version,
   int i;
   AttributeNode attr;
 
-  attribute = g_malloc(sizeof(Attribute));
+  attribute = g_malloc0(sizeof(Attribute));
   elem = &attribute->element;
-  obj = (Object *) attribute;
+  obj = &elem->object;
   
   obj->type = &attribute_type;
   obj->ops = &attribute_ops;
 
   element_load(elem, obj_node);
 
-  attribute->properties_dialog = NULL;
-
   attribute->border_width = 0.1;
   attr = object_find_attribute(obj_node, "border_width");
   if (attr != NULL)
@@ -656,6 +544,16 @@ static Object *attribute_load(ObjectNode obj_node, int version,
   if (attr != NULL)
     attribute->multivalue = data_boolean(attribute_first_data(attr));
 
+  attribute->font = NULL;
+  attr = object_find_attribute (obj_node, "font");
+  if (attr != NULL)
+	  attribute->font = data_font (attribute_first_data (attr));
+
+  attribute->font_height = FONT_HEIGHT;
+  attr = object_find_attribute (obj_node, "font_height");
+  if (attr != NULL)
+    attribute->font_height = data_real( attribute_first_data(attr) );
+
   element_init(elem, 8, 8);
 
   for (i=0;i<8;i++) {
@@ -664,15 +562,18 @@ static Object *attribute_load(ObjectNode obj_node, int version,
     attribute->connections[i].connected = NULL;
   }
 
-  attribute->font = font_getfont("Courier");
-
-  attribute->name_width = font_string_width(attribute->name, attribute->font, FONT_HEIGHT);
+  if (attribute->font == NULL) {
+	  /* choose default font name for your locale. see also font_data structure
+	     in lib/font.c. if "Courier" works for you, it would be better.  */
+	  attribute->font = font_getfont(_("Courier"));
+  }
 
+  attribute->name_width = font_string_width(attribute->name, attribute->font, attribute->font_height);
   attribute_update_data(attribute);
 
   for (i=0;i<8;i++) {
     obj->handles[i]->type = HANDLE_NON_MOVABLE;
   }
 
-  return (Object *)attribute;
+  return &attribute->element.object;
 }
