@@ -2,7 +2,7 @@
  * Copyright (C) 1998 Alexander Larsson
  *
  * GRAFCET chart support 
- * Copyright (C) 2000 Cyrille Chepelov
+ * Copyright (C) 2000,2001 Cyrille Chepelov
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -46,7 +46,9 @@
 #include "pixmaps/condition.xpm"
 
 #define CONDITION_LINE_WIDTH GRAFCET_GENERAL_LINE_WIDTH
-#define CONDITION_FONT "Helvetica-Bold"
+/* choose default font name for your locale. see also font_data structure
+   in lib/font.c. */
+#define CONDITION_FONT N_("Helvetica-Bold")
 #define CONDITION_FONT_HEIGHT 0.8
 #define CONDITION_LENGTH (1.5)
 #define CONDITION_ARROW_SIZE 0.0 /* XXX The norm says there's no arrow head.
@@ -58,7 +60,7 @@ typedef struct _Condition {
   Boolequation *cond;
 
   gchar *cond_value;
-  Font *cond_font;
+  DiaFont *cond_font;
   real cond_fontheight;
   Color cond_color;
 
@@ -83,9 +85,9 @@ static Object *condition_load(ObjectNode obj_node, int version,
 			       const char *filename);
 static PropDescription *condition_describe_props(Condition *condition);
 static void condition_get_props(Condition *condition, 
-                                Property *props, guint nprops);
+                                GPtrArray *props);
 static void condition_set_props(Condition *condition, 
-                                Property *props, guint nprops);
+                                GPtrArray *props);
 
 static ObjectTypeOps condition_type_ops =
 {
@@ -125,7 +127,7 @@ static ObjectOps condition_ops = {
 
 static PropDescription condition_props[] = {
   CONNECTION_COMMON_PROPERTIES,
-  { "condition",PROP_TYPE_STRING, PROP_FLAG_VISIBLE,
+  { "condition",PROP_TYPE_STRING, PROP_FLAG_VISIBLE|PROP_FLAG_DONT_MERGE,
     N_("Condition"),N_("The boolean equation of the condition")},
   { "cond_font",PROP_TYPE_FONT, PROP_FLAG_VISIBLE,
     N_("Font"),N_("The condition's font") },
@@ -156,17 +158,17 @@ static PropOffset condition_offsets[] = {
 };
 
 static void
-condition_get_props(Condition *condition, Property *props, guint nprops)
+condition_get_props(Condition *condition, GPtrArray *props)
 {  
   object_get_props_from_offsets(&condition->connection.object,
-                                condition_offsets,props,nprops);
+                                condition_offsets,props);
 }
 
 static void
-condition_set_props(Condition *condition, Property *props, guint nprops)
+condition_set_props(Condition *condition, GPtrArray *props)
 {
   object_set_props_from_offsets(&condition->connection.object,
-                                condition_offsets,props,nprops);
+                                condition_offsets,props);
   
   boolequation_set_value(condition->cond,condition->cond_value);
   condition->cond->font = condition->cond_font;
@@ -312,11 +314,11 @@ condition_create(Point *startpoint,
 {
   Condition *condition;
   Connection *conn;
-  ConnectionBBExtras *extra;
+  LineBBExtras *extra;
   Object *obj;
   Point defaultlen  = {0.0,CONDITION_ARROW_SIZE}, pos;
 
-  Font *default_font; 
+  DiaFont *default_font; 
   real default_fontheight;
   Color fg_color;
 
