@@ -29,20 +29,9 @@
 
 Color computer_color = { 0.7, 0.7, 0.7 };
 
-extern ObjectType computer_type;
-extern ObjectType monitor_type;
-extern ObjectType disc_type;
 extern ObjectType bus_type;
 extern ObjectType bus_type_std;
-extern ObjectType printer_type;
-extern ObjectType flash_type;
 extern ObjectType wanlink_type;
-extern ObjectType hub_type;
-extern ObjectType modularswitch_type;
-extern ObjectType rj45plug_type;
-extern ObjectType sceadplug_type;
-extern ObjectType modem_type;
-extern ObjectType antenna_type;
 
 DIA_PLUGIN_CHECK_INIT
 
@@ -53,20 +42,9 @@ dia_plugin_init(PluginInfo *info)
 			    NULL, NULL))
     return DIA_PLUGIN_INIT_ERROR;
 
-  object_register_type(&computer_type);
-  object_register_type(&monitor_type);
-  object_register_type(&disc_type);
   object_register_type(&bus_type_std);
   object_register_type(&bus_type);
-  object_register_type(&printer_type);
-  object_register_type(&flash_type);
   object_register_type(&wanlink_type);
-  object_register_type(&hub_type);
-  object_register_type(&modularswitch_type);
-  object_register_type(&rj45plug_type);
-  object_register_type(&sceadplug_type);
-  object_register_type(&modem_type);
-  object_register_type(&antenna_type);
 
   return DIA_PLUGIN_INIT_OK;
 }
